/**
  **************************************************************************
  * @file     touch.c
  * @version  v2.0.0
  * @date     2020-11-02
  * @brief    touch application libray header file.
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to 
  * download from Artery official website is the copyrighted work of Artery. 
  * Artery authorizes customers to use, copy, and distribute the BSP 
  * software and its related documentation for the purpose of design and 
  * development in conjunction with Artery microcontrollers. Use of the 
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "touch.h"
#include "string.h"
#include "at32f403a_407_board.h"



/**
  * @brief i2c address direction
  */
#define I2C_DIR_TRANSMIT                 0x00
#define I2C_DIR_RECEIVE                  0x01

/**
  * @brief touch functions definition
  */  
touch_dev_type touch_dev_struct = 
{
  touch_init,          /*!< function for init touch pin */
  touch_read_xy,       /*!< function for read data from touch */
  touch_scan,          /*!< function for scanning the screen */
  touch_adjust,        /*!< function for adjust the screen */
  TOUCH_SCAN_VERTICAL, /*!< touch scan direction */
  0, 
  0, 
}; 

uint8_t cmd_rdx = 0xd0; 
uint8_t cmd_rdy = 0x90; 

touch_dev_type *touch_struct; 

/* gt9147 parameter configuration table */
const uint8_t GT9147_CFG_TBL[]=
{ 
  0x62,0xE0,0x01,0x20,0x03,0x05,0x37,0xC0,0x01,0x08,
  0x28,0x0F,0x50,0x32,0x03,0x05,0x00,0x00,0x00,0x00,
  0x00,0x00,0x06,0x16,0x16,0x1F,0x14,0x89,0x28,0x0A,
  0x17,0x15,0x31,0x0D,0x00,0x00,0x08,0x22,0x04,0x11,
  0x00,0x00,0x00,0x00,0x00,0x03,0x82,0x08,0x08,0x00,
  0x00,0x0F,0x2C,0x94,0xC5,0x02,0x07,0x00,0x00,0x04,
  0x9D,0x10,0x00,0x84,0x14,0x00,0x70,0x19,0x00,0x5F,
  0x20,0x00,0x55,0x27,0x00,0x54,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,
  0x00,0x00,0x1A,0x18,0x16,0x14,0x12,0x10,0x0E,0x0C,
  0x0A,0x08,0x00,0x00,0x00,0x00,0x1F,0xFF,0xFF,0xFF,
  0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
  0xFF,0x00,0x00,0x02,0x04,0x05,0x06,0x08,0x0A,0x0C,
  0x0E,0x1D,0x1E,0x1F,0x20,0x22,0x24,0x28,0x29,0xFF,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0xFF,
  0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xF6,
  0xFF,0xFF,0xFF,0xFF,
};  

/* gt9147 point coordinate table */
const uint16_t GT9147_TPX_TBL[5] = {GT_TP1_REG, GT_TP2_REG, GT_TP3_REG, GT_TP4_REG, GT_TP5_REG};

/** 
  * @brief  i2c gpio initialization.
  * @param  none.
  * @retval none.
  */
void gt_i2c_init(void)
{
  gpio_init_type gpio_initstructure;
  
  /* i2c gpio clock enable */ 
  crm_periph_clock_enable(GT_SCL_GPIO_CLK, TRUE);
  crm_periph_clock_enable(GT_SDA_GPIO_CLK, TRUE);
  crm_periph_clock_enable(CRM_IOMUX_PERIPH_CLOCK, TRUE);  
  
  gpio_pin_remap_config(SWJTAG_MUX_010, TRUE);  
  
  I2C_SDA_HIGH();
  
  I2C_SCL_HIGH();
  
  /* gpio configuration */  
  gpio_initstructure.gpio_out_type       = GPIO_OUTPUT_OPEN_DRAIN;  
  gpio_initstructure.gpio_pull           = GPIO_PULL_UP;  
  gpio_initstructure.gpio_mode           = GPIO_MODE_OUTPUT;  
  gpio_initstructure.gpio_drive_strength = GPIO_DRIVE_STRENGTH_MODERATE;

  /* configure i2c pins: scl */    
  gpio_initstructure.gpio_pins = GT_SCL_GPIO_PIN;
  gpio_init(GT_SCL_GPIO_PORT, &gpio_initstructure);

  /* configure i2c pins: sda */     
  gpio_initstructure.gpio_pins = GT_SDA_GPIO_PIN;
  gpio_init(GT_SDA_GPIO_PORT, &gpio_initstructure);
}

/** 
  * @brief  used to set the i2c clock frequency.
  * @param  none.
  * @retval none.
  */
void i2c_delay(void)
{
  delay_us(5);
}

/** 
  * @brief  used to generate start conditions.
  * @param  none.
  * @retval none.
  */
void i2c_start(void)
{
  i2c_delay();
  
  I2C_SDA_HIGH();
  I2C_SCL_HIGH();
  i2c_delay();
  
  I2C_SDA_LOW();
  i2c_delay();
  
  I2C_SCL_LOW();
}

/** 
  * @brief  used to generate stop conditions.
  * @param  none.
  * @retval none.
  */
void i2c_stop(void)
{
  I2C_SCL_LOW();
  I2C_SDA_LOW();  
  i2c_delay();
  
  I2C_SCL_HIGH();
  i2c_delay();
  
  I2C_SDA_HIGH();
  i2c_delay();
}

/** 
  * @brief  used to generate ack conditions.
  * @param  none.
  * @retval none.
  */
void i2c_ack(void)
{
  I2C_SCL_LOW();
  I2C_SDA_LOW();
  i2c_delay();
  
  I2C_SCL_HIGH();
  i2c_delay();
  
  I2C_SCL_LOW();
}

/** 
  * @brief  used to generate nack conditions.
  * @param  none.
  * @retval none.
  */
void i2c_no_ack(void)
{
  I2C_SCL_LOW();
  I2C_SDA_HIGH();
  i2c_delay();
  
  I2C_SCL_HIGH();
  i2c_delay();
  
  I2C_SCL_LOW();
}

/** 
  * @brief  used to wait ack conditions.
  * @param  none.
  * @retval ack receive status.
  *         - ERROR: no ack received.
  *         - SUCCESS: ack received.
  */
error_status i2c_wait_ack(uint8_t timeout)
{
  I2C_SCL_LOW();
  I2C_SDA_HIGH();  
  
  i2c_delay();
  
  while(timeout)
  {
    if (I2C_SDA_READ() == 0)
    {
      I2C_SCL_HIGH(); 
      
      i2c_delay(); 
      
      I2C_SCL_LOW();
      
      return SUCCESS;
    }
    
    i2c_delay();
    
    timeout--;
  }  

  I2C_SCL_HIGH(); 
  i2c_delay();      

  I2C_SCL_LOW();
  
  i2c_delay();  
  
  return ERROR;
}

/** 
  * @brief  send a byte.
  * @param  data: byte to be transmitted.
  * @retval none.
  */
void i2c_send_byte(uint8_t data)
{
  uint8_t i = 8;
  
  while (i--)
  {
    I2C_SCL_LOW();
    
    if (data & 0x80)
    {
      I2C_SDA_HIGH();    
    }
    else
    {
      I2C_SDA_LOW();    
    }    
    
    i2c_delay();

    data <<= 1;
    
    I2C_SCL_HIGH();
    i2c_delay();
  }
  
  I2C_SCL_LOW();
  I2C_SDA_HIGH();    
}

/** 
  * @brief  receive a byte.
  * @param  data: byte to be received.
  * @retval none.
  */
uint8_t i2c_receive_byte(void)
{
  uint8_t i = 8;
  uint8_t byte = 0;

  I2C_SDA_HIGH();
  
  while (i--) 
  {
    byte <<= 1;
    
    I2C_SCL_LOW();
    i2c_delay();
    
    I2C_SCL_HIGH();
    i2c_delay();
    
    if (I2C_SDA_READ()) 
    {
      byte |= 0x01;
    }
  }
  
  I2C_SCL_LOW();
  
  return byte;
}

/** 
  * @brief  write data to the memory device through polling mode.
  * @param  address: memory device address.
  * @param  mem_address: memory address.
  * @param  data: data buffer.
  * @param  size: data size.
  * @param  timeout: maximum waiting time.
  * @retval function execution result.
  *         - SUCCESS.
  *         - ERROR.
  */
error_status gt_i2c_write(uint16_t address, uint16_t mem_address, uint8_t* data, uint8_t size, uint32_t timeout)
{
  uint16_t i;
  
  /* generate start condition */
  i2c_start();

  /* send device address */
  i2c_send_byte(address | I2C_DIR_TRANSMIT);

  if (i2c_wait_ack(GT_I2C_TIMEOUT) == ERROR)
  {
    /* generate stop condition */
    i2c_stop();
    
    return ERROR;
  }
  
  /* send memory address */
  i2c_send_byte(mem_address >> 8);

  if (i2c_wait_ack(GT_I2C_TIMEOUT) == ERROR)
  {
    /* generate stop condition */
    i2c_stop();
    
    return ERROR;
  }

  /* send memory address */
  i2c_send_byte(mem_address & 0xFF);

  if (i2c_wait_ack(GT_I2C_TIMEOUT) == ERROR)
  {
    /* generate stop condition */
    i2c_stop();
    
    return ERROR;
  }
  
  for (i = 0; i < size; i++) 
  {
    /* send data */
    i2c_send_byte(data[i]);
    
    if (i2c_wait_ack(GT_I2C_TIMEOUT) == ERROR)
    {
      /* generate stop condition */
      i2c_stop();
      
      return ERROR;
    }
  }

  /* generate stop condition */
  i2c_stop();
  
  return SUCCESS;
}

/** 
  * @brief  read data from memory device.
  * @param  address: memory device address.
  * @param  mem_address: memory address.
  * @param  data: data buffer.
  * @param  size: data size.
  * @param  timeout: maximum waiting time.
  * @retval function execution result.
  *         - SUCCESS.
  *         - ERROR.
  */
error_status gt_i2c_read(uint16_t address, uint16_t mem_address, uint8_t* data, uint8_t size, uint32_t timeout)
{
  uint16_t i;
  
  /* generate start condition */
  i2c_start();
  
  /* send device address */
  i2c_send_byte(address | I2C_DIR_TRANSMIT);

  /* waiting for ack */
  if (i2c_wait_ack(GT_I2C_TIMEOUT) == ERROR)
  {
    /* generate stop condition */
    i2c_stop();
    
    return ERROR;
  }
  
  /* send memory address */
  i2c_send_byte(mem_address >> 8);

  /* waiting for ack */
  if (i2c_wait_ack(GT_I2C_TIMEOUT) == ERROR)
  {
    i2c_stop();
    
    return ERROR;
  }

  /* send memory address */
  i2c_send_byte(mem_address & 0xFF);

  /* waiting for ack */
  if (i2c_wait_ack(GT_I2C_TIMEOUT) == ERROR)
  {
    /* generate stop condition */
    i2c_stop();
    
    return ERROR;
  }
  
  /* generate start condition */
  i2c_start();
  
  /* send device address */
  i2c_send_byte(address | I2C_DIR_RECEIVE);

  /* waiting for ack */
  if (i2c_wait_ack(GT_I2C_TIMEOUT) == ERROR)
  {
    /* generate stop condition */
    i2c_stop();
    
    return ERROR;
  }
  
  for (i = 0; i < size; i++) 
  {
    /* read data */
    data[i] = i2c_receive_byte();
    
    if (i < (size - 1))
    {
      i2c_ack();
    }
    else
    {
      i2c_no_ack(); 
    }
  }
  
  /* generate stop condition */
  i2c_stop();
  
  return SUCCESS;
}

/** 
  * @brief  gt9147 register write.
  * @param  reg: register address.
  * @param  buf: data buffer.
  * @param  len: data size.
  * @retval function execution result.
  *         - SUCCESS.
  *         - ERROR.
  */
error_status gt_reg_write(uint16_t reg, uint8_t* buf, uint8_t len)
{
  if(gt_i2c_write(GT_I2C_ADDRESS, reg, buf, len, GT_I2C_TIMEOUT) != SUCCESS)
  { 
    return ERROR;
  }
  else
  {
    return SUCCESS;
  }
}

/** 
  * @brief  gt9147 register read.
  * @param  reg: register address.
  * @param  buf: data buffer.
  * @param  len: data size.
  * @retval function execution result.
  *         - SUCCESS.
  *         - ERROR.
  */
error_status gt_reg_read(uint16_t reg, uint8_t* buf, uint8_t len)
{
  if(gt_i2c_read(GT_I2C_ADDRESS, reg, buf, len, GT_I2C_TIMEOUT) != SUCCESS)
  { 
    return ERROR;
  }
  else
  {
    return SUCCESS;
  }
} 

/** 
  * @brief  gt9147 configure all register.
  * @param  mode: parameter save mode.
  *         - 0: save to ram.
  *         - 1: save to flash(power-down save).
  * @retval function execution result.
  *         - SUCCESS.
  *         - ERROR.
  */
error_status gt_send_cfg(uint8_t mode)
{
  uint8_t buf[2];
  uint8_t i = 0;
  buf[0] = 0;
  buf[1] = mode;
  
  for(i = 0; i < sizeof(GT9147_CFG_TBL); i++)
  {
    /* calculate the checksum */
    buf[0] += GT9147_CFG_TBL[i];
  }

  buf[0] = (~buf[0]) + 1;
  
  /* write all register */
  gt_reg_write(GT_CFG_REG, (uint8_t*)GT9147_CFG_TBL, sizeof(GT9147_CFG_TBL));
  
  /* write checksum */
  gt_reg_write(GT_CHECK_REG, buf, 2);
  
  return SUCCESS;
} 

/**
  * @brief  this function is init touch.
  * @param  direction: touch scan direction
  *         this parameter can be one of the following values:
  *         - TOUCH_SCAN_VERTICAL: vertical scan.
  *         - TOUCH_SCAN_HORIZONTAL: horizontal scan.
  * @retval function execution result.
  *         - SUCCESS.
  *         - ERROR.
  */
error_status touch_init(touch_scan_type direction)
{
  error_status status = ERROR;
  uint8_t temp[5];  
  gpio_init_type  gpio_init_struct = {0}; 
  
  touch_dev_struct.direction = direction;
  
  /* enable the gpio clock */
  crm_periph_clock_enable(GT_RST_GPIO_CLK, TRUE); 
  crm_periph_clock_enable(GT_INT_GPIO_CLK, TRUE); 
  
  gpio_init_struct.gpio_mode = GPIO_MODE_OUTPUT; 
  gpio_init_struct.gpio_out_type = GPIO_OUTPUT_PUSH_PULL; 
  gpio_init_struct.gpio_pull = GPIO_PULL_NONE; 
  gpio_init_struct.gpio_drive_strength = GPIO_DRIVE_STRENGTH_STRONGER; 

  GT_RST_LOW();
  gpio_init_struct.gpio_pins = GT_RST_GPIO_PIN;   
  gpio_init(GT_RST_GPIO_PORT, &gpio_init_struct); 
  
  GT_INT_HIGH();
  gpio_init_struct.gpio_pins = GT_INT_GPIO_PIN;   
  gpio_init(GT_INT_GPIO_PORT, &gpio_init_struct); 
  
  /* init i2c */
  gt_i2c_init();        
  
  /* set RST pin low */
  GT_RST_LOW();          

  delay_ms(10);
  
  /* set RST pin high */
  GT_RST_HIGH();        
  
  delay_ms(10);
 
  /* set INT pin low */
  GT_INT_LOW();
  
  gpio_init_struct.gpio_mode = GPIO_MODE_INPUT; 
  gpio_init_struct.gpio_pins = GT_INT_GPIO_PIN;   
  gpio_init(GT_INT_GPIO_PORT, &gpio_init_struct); 

  delay_ms(100);  

  /* read touch id */
  gt_reg_read(GT_PID_REG, temp, 4);

  temp[4] = 0;
  
  //printf("CTP ID:%s\r\n", temp);  
  
  if(strcmp((char*)temp, "9147") == 0)
  {
    temp[0] = 0x02;    
    
    /* software reset gt9147 */
    gt_reg_write(GT_CTRL_REG, temp, 1);
    
    /* read GT_CFG_REG register */
     gt_reg_read(GT_CFG_REG, temp, 1);

    /* if the default version is relatively low, update the flash configuration */
    if(temp[0] <= 0x62)
    {
      //printf("Default Ver:0x%X\r\n", temp[0]);
      
      /* update and save configuration */
      gt_send_cfg(1);
    }
    
    delay_ms(10);
    
    temp[0] = 0x00;   
    
    /* end software reset */
    gt_reg_write(GT_CTRL_REG, temp, 1);  

    status = SUCCESS;  
  } 
  
  return status;
}

/**
  * @brief  this function is read data from touch.
  * @param  x/y : coordinate vaule.
  * @retval function execution result.
  *         - SUCCESS.
  *         - ERROR.
  */
error_status touch_read_xy(uint16_t *x, uint16_t *y)
{
  uint8_t buf[4];
  uint8_t i = 0;
  uint8_t temp = 0;
  uint8_t mode, num = 0;  
  error_status status = ERROR;
  
  if(GT_INT_READ())
  {
    /* read touch point status */
    gt_reg_read(GT_STS_REG, &mode, 1);
    
    num = (mode & 0xF);  

    if((mode & 0x80) && (num < 6))
    {
      /* clear touch flag */
      gt_reg_write(GT_STS_REG, &temp, 1);  

      for(i = 0; i < num; i++)
      {
        /* read x y coordinates */
        gt_reg_read(GT9147_TPX_TBL[i], buf, 4); 

        if(touch_dev_struct.direction == TOUCH_SCAN_HORIZONTAL)
        {
          /* calculate actual coordinates */
          y[i] = (((uint16_t)buf[1] << 8) + buf[0]);
          x[i] = 800 - (((uint16_t)buf[3] << 8) + buf[2]);        
        }
        else
        {
          /* calculate actual coordinates */
          x[i] = (((uint16_t)buf[1] << 8) + buf[0]);
          y[i] = (((uint16_t)buf[3] << 8) + buf[2]);           
        }
      }
    } 

    if(num) 
    {
      status = SUCCESS;
    }
  }

  return status;
}

/**
  * @brief  this function is scanning the screen.
  * @param  none
  * @retval none
  */
void touch_scan(void)
{

}

/**
  * @brief  this function is adjust the screen.
  * @param  none
  * @retval state
  */
error_status touch_adjust(void)
{
  /* touch adjust code */
  return SUCCESS; 
}

